#!/usr/bin/perl
#===============================================================================
#
#         FILE:  sr_inversion.pl
#
#        USAGE:  perl sr_inversion.pl <options>
#
#  DESCRIPTION:__Split reads for inversion detection
#
#
#       AUTHOR:  Ruoyan Chen (cry), chenry@hku.hk
#      COMPANY:  The University of Hong Kong
#      VERSION:  1.0
#      CREATED:  Monday, March 17, 2014 03:53:02 HKT
# 
#===============================================================================


use warnings;
use strict;
use Getopt::Long;
use Pod::Usage;

our ($man, $help);
our ($scenario,$step,$chr,$keep);
our ($input,$outpre);
our ($bwa, $samtools, $reference,$path);

our ($clip,$gap,$match);

GetOptions('help|h'=>\$help, 'man|m'=>\$man, 'input|i=s'=>\$input, 'outprefix|o=s'=>\$outpre, 'scenario|s=s'=>\$scenario, 'step|t=s'=>\$step, 'bwa|b=s'=>\$bwa, 'samtools|l=s'=>\$samtools, 'reference|r=s'=>\$reference, 'chr|c=s'=>\$chr, 'keep|k'=>\$keep, 'path|p=s'=>\$path, 'clip|cp=i'=>\$clip, 'gap|g=i'=>\$gap, 'match|m'=>\$match) or pod2usage (1);

$help and pod2usage (-verbose=>1, -exitval=>1, -output=>\*STDOUT);
$man and pod2usage (-verbose=>2, -exitval=>1, -output=>\*STDOUT);

pod2usage ("-i,--input is required!\n") if not defined $input;
pod2usage ("-o,--outpre is required!\n") if not defined $outpre;
pod2usage ("-s,--scenario is required!\n") if not defined $scenario;
pod2usage ("-r,--reference is required!\n") if not defined $reference;

if (not defined $path){
	$path = $0;
	$path =~ s/SRinversion\.pl//;
}

my $lib;
$lib = "$path/lib/";

if (not defined $bwa){
	$bwa = "$path/pub/bwa";
}

if (not defined $samtools){
	$samtools = "$path/pub/samtools";
}

if (not defined $clip){
	$clip = 10;
}

if (not defined $gap){
	$gap = 1;
}

print "\n\n";
print "*********************************************************\n";
print "Welcome to use SRinversion for short inversion detection!\n";
print "*********************************************************\n";
print "\n\nStart scenario $scenario now...\n\n";


############Senaria 1################
#####################################
if ((not defined $scenario)||($scenario eq "1")){

	######Step1######
	if ((not defined $step)||($step eq "1")){
			open IA, "$samtools view $input|"||die$!;
			open OA, ">$outpre.low_quality.sam"||die$!;
			while (<IA>){
				chomp;
				my @line = split/\s+/,$_;
				my $clip_end_len = 0;
				if ($line[5] =~ /^(\d+)S/){
					$clip_end_len = $1;
				}
				if ($line[5] =~ /(\d+)S$/){
					if (!$clip_end_len){
						$clip_end_len = $1;
					}elsif ($clip_end_len < $1){
						$clip_end_len = $1;
					}
				}
				if (defined $chr){
					$line[2] =~ s/chr//g;
					$line[2] =~ s/Chr//g;
					if (($line[2] eq $chr) && ($line[1] < 256) && ($clip_end_len >= $clip)){
						print OA "$_\n";
					}
				}else {
					if (($line[1] < 256) && ($clip_end_len >= $clip)){
						print OA "$_\n";
					}
				}
			}
			close OA;
			close IA;
		
		print "SENARIA 1 STEP1 finished\n";
		system("date");
		print "\n\n";
	}

	######Step2######
	if ((not defined $step)||($step eq "2")){
		system("perl $lib/scenario1_step2.pl $outpre.low_quality.sam $outpre.low_quality.fq $gap");
		system("gzip $outpre.low_quality.fq");
		if (not defined $keep){
			system("rm $outpre.low_quality.sam");
		}
		print "SENARIA 1 STEP2 finished\n";
		system("date");
		print "\n\n";
	}

	######Step3######
	if ((not defined $step)||($step eq "3")){
		system("$bwa aln $reference -t 8 $outpre.low_quality.fq.gz > $outpre.low_quality.fq.sai 2>$outpre.sr.sam.log");
		system("$bwa samse $reference $outpre.low_quality.fq.sai $outpre.low_quality.fq.gz > $outpre.sr.sam 2>> $outpre.sr.sam.log");
		if (not defined $keep){
			system("rm $outpre.low_quality.fq.gz $outpre.low_quality.fq.sai");
		}
	
		print "SENARIA 1 STEP3 finished\n";
		system("date");
		print "\n\n";
	}
	
	######Step4######
	if ((not defined $step)||($step eq "4")){
		if (not defined $chr){
			for my $chri ((1..22),"X","Y","M"){
				if (defined $match){
					system("perl $lib/scenario1_step4.pl $outpre.sr.sam $outpre.sr.sam.$chri $chri match");
				}else {
					system("perl $lib/scenario1_step4.pl $outpre.sr.sam $outpre.sr.sam.$chri $chri not");
				}
			}
		}else {
			if (defined $match){
				system("perl $lib/scenario1_step4.pl $outpre.sr.sam $outpre.sr.sam.$chr $chr match");
			}else {
				system("perl $lib/scenario1_step4.pl $outpre.sr.sam $outpre.sr.sam.$chr $chr not");
			}
		}
		if (not defined $keep){
			system("rm $outpre.sr.sam");
		}
		print "SENARIA 1 STEP4 finished\n";
		system("date");
		print "\n\n";
	}

	######Step5######
	if ((not defined $step)||($step eq "5")){
		if (not defined $chr){
			for my $chri ((1..22),"X","Y","M"){
				system("perl $lib/scenario1_step5.pl $outpre.sr.sam.$chri $outpre.sr.sam.$chri.inversion $chri");
			}
		}else {
			system("perl $lib/scenario1_step5.pl $outpre.sr.sam.$chr $outpre.sr.sam.$chr.inversion $chr");
		}
		if (not defined $keep){
			if (not defined $chr){
				for my $chri ((1..22),"X","Y","M"){
					system("rm $outpre.sr.sam.$chri");
				}
			}else {
				system("rm $outpre.sr.sam.$chr");
			}
		}
		print "SENARIA 1 STEP5 finished\n";
		system("date");
		print "\n\n";
	}

	######Step6######
	if ((not defined $step)||($step eq "6")){
		if (not defined $chr){
			for my $chri ((1..22),"X","Y","M"){
				system("perl $lib/scenario1_step6.pl $outpre.sr.sam.$chri.inversion $outpre.sr.sam.$chri.inversion.merge $chri");
			}
		}else {
			system("perl $lib/scenario1_step6.pl $outpre.sr.sam.$chr.inversion $outpre.sr.sam.$chr.inversion.merge $chr");
		}
		if (not defined $keep){
			if (not defined $chr){
				for my $chri ((1..22),"X","Y","M"){
					system("rm $outpre.sr.sam.$chri.inversion");
				}
			}else {
				system("rm $outpre.sr.sam.$chr.inversion");
			}
		}
		print "SENARIA 1 STEP6 finished\n";
		system("date");
		print "\n\n";
	}
	print "***********************************************\n";
	print "SENARIA 1 all done!\nWish you a happy day! ~.~\n";
	print "***********************************************\n";



############Senaria 1################
#####################################
}elsif ((not defined $scenario)||($scenario eq "2")){

        ######Step1-2######
        if ((not defined $step)||($step eq "1")||($step eq "2")){
		if (not defined $chr){
                	system("perl $lib/scenario2_step1-2.pl $input $outpre.low_quality.fq all $samtools $gap");
		}else {
			system("perl $lib/scenario2_step1-2.pl $input $outpre.low_quality.fq $chr $samtools $gap");
		}
		system("gzip $outpre.low_quality.fq");
                print "SENARIA 2 STEP1-2 finished\n";
                system("date");
	}

        ######Step3######
        if ((not defined $step)||($step eq "3")){
                system("$bwa aln $reference -t 8 $outpre.low_quality.fq.gz > $outpre.low_quality.fq.sai 2>$outpre.sr.sam.log");
                system("$bwa samse $reference $outpre.low_quality.fq.sai $outpre.low_quality.fq.gz > $outpre.sr.sam 2>> $outpre.sr.sam.log");
                if (not defined $keep){
                        system("rm $outpre.low_quality.fq.gz $outpre.low_quality.fq.sai");
                }
                print "SENARIA 2 STEP3 finished\n";
                system("date");
        }

        ######Step4######
        if ((not defined $step)||($step eq "4")||($step eq "5")){
                if (not defined $chr){
                        for my $chri ((1..22),"X","Y","M"){
                                if (defined $match){
					system("perl $lib/scenario2_step4-5.pl $outpre.sr.sam $outpre.sr.sam.$chri.inversion $chri match");
				}else {
					system("perl $lib/scenario2_step4-5.pl $outpre.sr.sam $outpre.sr.sam.$chri.inversion $chri not");
				}
                        }
                }else {
			if (defined $match){
                        	system("perl $lib/scenario2_step4-5.pl $outpre.sr.sam $outpre.sr.sam.$chr.inversion $chr match");
			}else {
				system("perl $lib/scenario2_step4-5.pl $outpre.sr.sam $outpre.sr.sam.$chr.inversion $chr not");
			}
                }
                if (not defined $keep){
                        system("rm $outpre.sr.sam");
                }
                print "SENARIA 2 STEP4-5 finished\n";
                system("date");
        }
        ######Step6######
        if ((not defined $step)||($step eq "6")){
                if (not defined $chr){
                        for my $chri ((1..22),"X","Y","M"){
                                system("perl $lib/scenario2_step6.pl $outpre.sr.sam.$chri.inversion $outpre.sr.sam.$chri.inversion.merge $chri");
                        }
                }else {
                        system("perl $lib/scenario2_step6.pl $outpre.sr.sam.$chr.inversion $outpre.sr.sam.$chr.inversion.merge $chr");
                }
                if (not defined $keep){
                        if (not defined $chr){
                                for my $chri ((1..22),"X","Y","M"){
                                        system("rm $outpre.sr.sam.$chri.inversion");
                                }
                        }else {
                                system("rm $outpre.sr.sam.$chr.inversion");
                        }
                }
                print "SENARIA 2 STEP6 finished\n";
                system("date");
        }
        print "SENARIA 2 all done!\nWish you a happy day! ~.~\n";




}else {
	pod2usage ("wrong scenario: only 1 or 2 is accepted\n");
}



=head1 NAME

 SRinversion combined version

=head1 SYNOPSIS

 perl sr_inversion.pl [options]
 Options:
 -h,--help            brief help message
 -m,--man             full documentation


=head1 OPTIONS

=over 8

=item B<-h,--help>

Print a brief help message and exits.

=item B<-m,--man>

Prints the manual page and exits.

=item B<-i,--input>

Input bam file (Required)

=item B<-o,--outpre>

Prefix of output file (Required)

=item B<-s,--scenario>

Which scenario to use (Required):
	scenario 1: inversion partially covered by read
	scenario 2: inversion completely covered by read

=item B<-t,--step>

Which step to run only (all steps will be processed by default):
	step1:	Extract low quality reads
	step2:	Split reads
	step3:	re-map splitted reads
	step4:	Select candidate reads
	step5:	Detect inversion regions
	step6:	Combine inversion regions

=item B<-c,--chr>

Limited on which chromosome, all chromosomes (1..2,X,Y,M) will be used by default.

=item B<-b,--bwa>

Location of BWA for mapping

=item B<-l,--samtools>

Location of samtools for bam/sam files handling

=item B<-r,--reference>

Location of references in FASTA format (Required)

=item B<-k,--keep>

Keep all intermediate files. (Will generate large files, caution when using whole genome high-coverage data>

=item B<-p,--path>

Path of libraries.

=item B<-cp,--clip>

Only reads with longer than #bp clipped ends will be included for scenario 1 [10]

=item B<-g,--gap>

#bp gap length for slideing window to move on each time when splitting reads [1]

=item B<-m,--match>

Only select exact matched reads

=back

=head1 DESCRIPTION

SRinversion uses Split reads method to detect short inversions (2bp ~ 1kb) from short reads in bam format.

=cut

